@@ -258,7 +258,7 @@ int cbstrbwmatch(const char *str, const char *key){
   slen = strlen(str);
   klen = strlen(key);
   for(i = 1; i <= klen; i++){
-    if(str[slen-i] != key[klen-i] || i > slen) return FALSE;
+    if(i > slen || str[slen-i] != key[klen-i]) return FALSE;
   }
   return TRUE;
 }
@@ -271,16 +271,81 @@ int cbstrbwimatch(const char *str, const char *key){
   slen = strlen(str);
   klen = strlen(key);
   for(i = 1; i <= klen; i++){
+    if(i > slen) return FALSE;
     sc = str[slen-i];
     if(sc >= 'A' && sc <= 'Z') sc += 'a' - 'A';
     kc = key[klen-i];
     if(kc >= 'A' && kc <= 'Z') kc += 'a' - 'A';
-    if(sc != kc || i > slen) return FALSE;
+    if(sc != kc) return FALSE;
   }
   return TRUE;
 }
 
 
+/* Locate a substring in a string using KMP method. */
+char *cbstrstrkmp(const char *haystack, const char *needle){
+  int i, j, hlen, nlen;
+  char tbl[0x100];
+  assert(haystack && needle);
+  nlen = strlen(needle);
+  if(nlen >= 0x100) return strstr(haystack, needle);
+  tbl[0] = -1;
+  i = 0;
+  j = -1;
+  while(i < nlen){
+    while((j >= 0) && (needle[i] != needle[j])){
+      j = tbl[j];
+    }
+    i++;
+    j++;
+    tbl[i] = j;
+  }
+  hlen = strlen(haystack);
+  i = 0;
+  j = 0;
+  while(i < hlen && j < nlen){
+    while((j >= 0) && (haystack[i] != needle[j])){
+      j = tbl[j];
+    }
+    i++;
+    j++;
+  }
+  if(j == nlen) return (char *)(haystack + i - nlen);
+  return NULL;
+}
+
+
+/* Locate a substring in a string using BM method. */
+char *cbstrstrbm(const char *haystack, const char *needle){
+  const unsigned char *rp;
+  const char *ep;
+  unsigned char tbl[0x100];
+  int i, j, nlen, len, idx;
+  assert(haystack && needle);
+  nlen = strlen(needle);
+  if(nlen < 3 || nlen >= 0x100) return strstr(haystack, needle);
+  for(i = 0; i < 0x100; i++){
+    tbl[i] = nlen;
+  }
+  len = nlen;
+  rp = (const unsigned char *)needle;
+  while(len > 0){
+    tbl[*rp++] = --len;
+  }
+  nlen--;
+  ep = haystack + strlen(haystack) - nlen;
+  while(haystack < ep){
+    for(i = nlen; haystack[i] == needle[i]; i--){
+      if(i == 0) return (char *)haystack;
+    }
+    idx = ((unsigned char *)haystack)[i];
+    j = tbl[idx] - nlen + i;
+    haystack += j > 0 ? j : 2;
+  }
+  return NULL;
+}
+
+
 /* Convert the letters of a string to upper case. */
 char *cbstrtoupper(char *str){
   int i;
@@ -2269,7 +2334,7 @@ char *cbxmlunescape(const char *str){
 char *cbdeflate(const char *ptr, int size, int *sp){
   assert(ptr && sp);
   if(!_qdbm_deflate) return NULL;
-  return _qdbm_deflate(ptr, size, sp);
+  return _qdbm_deflate(ptr, size, sp, _QDBM_ZMZLIB);
 }
 
 
@@ -2277,7 +2342,23 @@ char *cbdeflate(const char *ptr, int size, int *sp){
 char *cbinflate(const char *ptr, int size, int *sp){
   assert(ptr && size >= 0);
   if(!_qdbm_inflate) return NULL;
-  return _qdbm_inflate(ptr, size, sp);
+  return _qdbm_inflate(ptr, size, sp, _QDBM_ZMZLIB);
+}
+
+
+/* Compress a serial object with GZIP. */
+char *cbgzencode(const char *ptr, int size, int *sp){
+  assert(ptr && sp);
+  if(!_qdbm_deflate) return NULL;
+  return _qdbm_deflate(ptr, size, sp, _QDBM_ZMGZIP);
+}
+
+
+/* Decompress a serial object compressed with GZIP. */
+char *cbgzdecode(const char *ptr, int size, int *sp){
+  assert(ptr && size >= 0);
+  if(!_qdbm_inflate) return NULL;
+  return _qdbm_inflate(ptr, size, sp, _QDBM_ZMGZIP);
 }
 
 
