@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The utitlity API of QDBM
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -23,6 +23,7 @@ extern "C" {
 
 
 #include <stdlib.h>
+#include <time.h>
 
 
 
@@ -66,6 +67,7 @@ typedef struct {                         /* type of structure for a map */
   CBMAPDATUM *first;                     /* pointer to the first element */
   CBMAPDATUM *last;                      /* pointer to the last element */
   CBMAPDATUM *cur;                       /* pointer to the current element */
+  int bnum;                              /* number of buckets */
   int rnum;                              /* number of records */
 } CBMAP;
 
@@ -205,6 +207,24 @@ int cbstrbwmatch(const char *str, const char *key);
 int cbstrbwimatch(const char *str, const char *key);
 
 
+/* Locate a substring in a string using KMP method.
+   `haystack' specifies the pointer of a target string.
+   `needle' specifies the pointer of a substring to be found.
+   The return value is the pointer to the beginning of the substring or `NULL' if the substring
+   is not found.
+   In most cases, `strstr' as a built-in function of the compiler is faster than this function. */
+char *cbstrstrkmp(const char *haystack, const char *needle);
+
+
+/* Locate a substring in a string using BM method.
+   `haystack' specifies the pointer of a target string.
+   `needle' specifies the pointer of a substring to be found.
+   The return value is the pointer to the beginning of the substring or `NULL' if the substring
+   is not found.
+   In most cases, `strstr' as a built-in function of the compiler is faster than this function. */
+char *cbstrstrbm(const char *haystack, const char *needle);
+
+
 /* Convert the letters of a string to upper case.
    `str' specifies the pointer of a string to convert.
    The return value is the pointer to the string. */
@@ -217,12 +237,18 @@ char *cbstrtoupper(char *str);
 char *cbstrtolower(char *str);
 
 
-/* Cut the space characters at head or tail of a string.
+/* Cut space characters at head or tail of a string.
    `str' specifies the pointer of a string to convert.
    The return value is the pointer to the string. */
 char *cbstrtrim(char *str);
 
 
+/* Squeeze space characters in a string and trim it.
+   `str' specifies the pointer of a string to convert.
+   The return value is the pointer to the string. */
+char *cbstrsqzspc(char *str);
+
+
 /* Get a datum handle.
    `ptr' specifies the pointer to the region of the initial content.  If it is `NULL', an empty
    datum is created.
@@ -384,6 +410,16 @@ void cblistinsert(CBLIST *list, int index, const char *ptr, int size);
 char *cblistremove(CBLIST *list, int index, int *sp);
 
 
+/* Overwrite an element at the specified location of a list.
+   `list' specifies a list handle.
+   `index' specifies the index of an element.
+   `ptr' specifies the pointer to the region of the new content.
+   `size' specifies the size of the new content.  If it is negative, the size is assigned with
+   `strlen(ptr)'.
+   If `index' is equal to or more than the number of elements, this function has no effect. */
+void cblistover(CBLIST *list, int index, const char *ptr, int size);
+
+
 /* Sort elements of a list in lexical order.
    `list' specifies a list handle.
    Quick sort is used for sorting. */
@@ -458,6 +494,18 @@ void cbmapclose(CBMAP *map);
 int cbmapput(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz, int over);
 
 
+/* Concatenate a value at the end of the value of the existing record.
+   `map' specifies a map handle.
+   `kbuf' specifies the pointer to the region of a key.
+   `ksiz' specifies the size of the region of the key.  If it is negative, the size is assigned
+   with `strlen(kbuf)'.
+   `vbuf' specifies the pointer to the region of a value.
+   `vsiz' specifies the size of the region of the value.  If it is negative, the size is
+   assigned with `strlen(vbuf)'.
+   If there is no corresponding record, a new record is created. */
+void cbmapputcat(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz);
+
+
 /* Delete a record.
    `map' specifies a map handle.
    `kbuf' specifies the pointer to the region of a key.
@@ -550,6 +598,21 @@ char *cbmapdump(CBMAP *map, int *sp);
 CBMAP *cbmapload(const char *ptr, int size);
 
 
+/* Extract a record from a serialized map.
+   `ptr' specifies the pointer to a byte array.
+   `size' specifies the size of the region.
+   `kbuf' specifies the pointer to the region of a key.
+   `ksiz' specifies the size of the region of the key.  If it is negative, the size is assigned
+   with `strlen(kbuf)'.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.  If it is `NULL', it is not used.
+   If successful, the return value is the pointer to the region of the value of the
+   corresponding record.  `NULL' is returned when no record corresponds.
+   Because an additional zero code is appended at the end of the region of the return value,
+   the return value can be treated as a character string. */
+char *cbmaploadone(const char *ptr, int size, const char *kbuf, int ksiz, int *sp);
+
+
 /* Allocate a formatted string on memory.
    `format' specifies a printf-like format string.  The conversion character `%' can be used
    with such flag characters as `d', `o', `u', `x', `X', `e', `E', `f', `g', `G', `c', `s', and
@@ -587,7 +650,7 @@ CBLIST *cbsplit(const char *ptr, int size, const char *delim);
 
 
 /* Read whole data of a file.
-   `name' specifies the name of a file.
+   `name' specifies the name of a file.  If it is `NULL', the standard input is specified.
    `sp' specifies the pointer to a variable to which the size of the region of the return
    value is assigned.  If it is `NULL', it is not used.
    The return value is the pointer to the allocated region of the read data.  Because an
@@ -599,7 +662,7 @@ char *cbreadfile(const char *name, int *sp);
 
 
 /* Write a serial datum into a file.
-   `name specifies the name of a file.
+   `name specifies the name of a file.  If it is `NULL', the standard output is specified.
    `ptr' specifies the pointer to the region of the source content.
    `size' specifies the size of the region.  If it is negative, the size is assigned with
    `strlen(ptr)'.
@@ -609,15 +672,18 @@ int cbwritefile(const char *name, const char *ptr, int size);
 
 
 /* Read every line of a file.
-   `name' specifies the name of a file.
-   The return value is a list handle of the lines if successful, else it is NULL.
-   Line separators are cut out. */
+   `name' specifies the name of a file.  If it is `NULL', the standard input is specified.
+   The return value is a list handle of the lines if successful, else it is NULL.  Line
+   separators are cut out.  Because the handle of the return value is opened with the function
+   `cblistopen', it should be closed with the function `cblistclose' if it is no longer in use. */
 CBLIST *cbreadlines(const char *name);
 
 
 /* Read names of files in a directory.
    `name' specifies the name of a directory.
-   The return value is a list handle of names if successful, else it is NULL. */
+   The return value is a list handle of names if successful, else it is NULL.
+   Because the handle of the return value is opened with the function `cblistopen', it should
+   be closed with the function `cblistclose' if it is no longer in use. */
 CBLIST *cbdirlist(const char *name);
 
 
@@ -715,27 +781,33 @@ char *cbquotedecode(const char *str, int *sp);
 
 
 /* Split a string of MIME into headers and the body.
-   `str' specifies the pointer to the region of a MIME string.
+   `ptr' specifies the pointer to the region of MIME data.
+   `size' specifies the size of the region.  If it is negative, the size is assigned with
+   `strlen(ptr)'.
    `attrs' specifies a map handle to store attributes.  If it is `NULL', it is not used.  Each
    key of the map is an attribute name uncapitalized.
-   The return value is the pointer of the body string.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.  If it is `NULL', it is not used.
+   The return value is the pointer of the body data.
    If the content type is defined, the attribute map has the key "TYPE" specifying the type.  If
    the character encoding is defined, the key "CHARSET" specifies the encoding name.  If the
    boundary string of multipart is defined, the key "BOUNDARY" specifies the string.  If the
    content disposition is defined, the key "DISPOSITION" specifies the direction.  If the file
-   name is defined, the key "FILENAME" specifies the name.  Because the region of the return
-   value is allocated with the `malloc' call, it should be released with the `free' call if
-   it is no longer in use. */
-char *cbmimebreak(const char *str, CBMAP *attrs);
+   name is defined, the key "FILENAME" specifies the name.  If the attribute name is defined,
+   the key "NAME" specifies the name.  Because the region of the return value is allocated with
+   the `malloc' call, it should be released with the `free' call if it is no longer in use. */
+char *cbmimebreak(const char *ptr, int size, CBMAP *attrs, int *sp);
 
 
-/* Split a string of multipart data of MIME into its parts.
-   `str' specifies the pointer to the region of a string of multipart data.
+/* Split multipart data of MIME into its parts.
+   `ptr' specifies the pointer to the region of multipart data of MIME.
+   `size' specifies the size of the region.  If it is negative, the size is assigned with
+   `strlen(ptr)'.
    `boundary' specifies the pointer to the region of the boundary string.
    The return value is a list handle.  Each element of the list is the string of a part.
    Because the handle of the return value is opened with the function `cblistopen', it should
    be closed with the function `cblistclose' if it is no longer in use. */
-CBLIST *cbmimeparts(const char *str, const char *boundary);
+CBLIST *cbmimeparts(const char *ptr, int size, const char *boundary);
 
 
 /* Encode a string with MIME encoding.
@@ -771,7 +843,8 @@ CBLIST *cbcsvrows(const char *str);
 
 /* Split the string of a row of CSV into cells.
    `str' specifies the pointer to the region of a row of CSV.
-   The return value is a list handle.  Each element of the list is a cell of the row.
+   The return value is a list handle.  Each element of the list is the unescaped string of a
+   cell of the row.
    Because the handle of the return value is opened with the function `cblistopen', it should
    be closed with the function `cblistclose' if it is no longer in use. */
 CBLIST *cbcsvcells(const char *str);
@@ -808,8 +881,8 @@ CBLIST *cbxmlbreak(const char *str, int cr);
 
 /* Get the map of attributes of an XML tag.
    `str' specifies the pointer to the region of a tag string.
-   The return value is a map handle.  Each key of the map is the name of an attribute.  You can
-   get the name of the tag with the key of an empty string.
+   The return value is a map handle.  Each key of the map is the name of an attribute.  Each
+   value is unescaped.  You can get the name of the tag with the key of an empty string.
    Because the handle of the return value is opened with the function `cbmapopen', it should
    be closed with the function `cbmapclose' if it is no longer in use. */
 CBMAP *cbxmlattrs(const char *str);
@@ -859,6 +932,41 @@ char *cbdeflate(const char *ptr, int size, int *sp);
 char *cbinflate(const char *ptr, int size, int *sp);
 
 
+/* Compress a serial object with GZIP.
+   `ptr' specifies the pointer to a region.
+   `size' specifies the size of the region.  If it is negative, the size is assigned with
+   `strlen(ptr)'.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.
+   If successful, the return value is the pointer to the result object, else, it is `NULL'.
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call if it is no longer in use.  This function is available only if
+   QDBM was built with ZLIB enabled. */
+char *cbgzencode(const char *ptr, int size, int *sp);
+
+
+/* Decompress a serial object compressed with GZIP.
+   `ptr' specifies the pointer to a region.
+   `size' specifies the size of the region.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.  If it is `NULL', it is not used.
+   If successful, the return value is the pointer to the result object, else, it is `NULL'.
+   Because an additional zero code is appended at the end of the region of the return value,
+   the return value can be treated as a character string.  Because the region of the return
+   value is allocated with the `malloc' call, it should be released with the `free' call if it
+   is no longer in use.  This function is available only if QDBM was built with ZLIB enabled. */
+char *cbgzdecode(const char *ptr, int size, int *sp);
+
+
+/* Get the CRC32 checksum of a serial object.
+   `ptr' specifies the pointer to a region.
+   `size' specifies the size of the region.  If it is negative, the size is assigned with
+   `strlen(ptr)'.
+   The return value is the CRC32 checksum of the object.
+   This function is available only if QDBM was built with ZLIB enabled. */
+unsigned int cbgetcrc(const char *ptr, int size);
+
+
 /* Convert the character encoding of a string.
    `ptr' specifies the pointer to a region.
    `size' specifies the size of the region.  If it is negative, the size is assigned with
@@ -888,6 +996,62 @@ char *cbiconv(const char *ptr, int size, const char *icode, const char *ocode, i
 const char *cbencname(const char *ptr, int size);
 
 
+/* Get the jet lag of the local time in seconds.
+   The return value is the jet lag of the local time in seconds. */
+int cbjetlag(void);
+
+
+/* Get the Gregorian calendar of a time.
+   `t' specifies a source time.  If it is negative, the current time is specified.
+   `jl' specifies the jet lag of a location in seconds.
+   `yearp' specifies the pointer to a variable to which the year is assigned.  If it is `NULL',
+   it is not used.
+   `monp' specifies the pointer to a variable to which the month is assigned.  If it is `NULL',
+   it is not used.  1 means January and 12 means December.
+   `dayp' specifies the pointer to a variable to which the day of the month is assigned.  If it
+   is `NULL', it is not used.
+   `hourp' specifies the pointer to a variable to which the hours is assigned.  If it is `NULL',
+   it is not used.
+   `minp' specifies the pointer to a variable to which the minutes is assigned.  If it is `NULL',
+   it is not used.
+   `secp' specifies the pointer to a variable to which the seconds is assigned.  If it is `NULL',
+   it is not used. */
+void cbcalendar(time_t t, int jl, int *yearp, int *monp, int *dayp,
+                int *hourp, int *minp, int *secp);
+
+
+/* Get the day of week of a date.
+   `year' specifies the year of a date.
+   `mon' specifies the month of the date.
+   `day' specifies the day of the date.
+   The return value is the day of week of the date.  0 means Sunday and 6 means Saturday. */
+int cbdayofweek(int year, int mon, int day);
+
+
+/* Get the string for a date in W3CDTF.
+   `t' specifies a source time.  If it is negative, the current time is specified.
+   `jl' specifies the jet lag of a location in seconds.
+   The return value is the string of the date in W3CDTF (YYYY-MM-DDThh:mm:ddTZD).
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call if it is no longer in use. */
+char *cbdatestrwww(time_t t, int jl);
+
+
+/* Get the string for a date in RFC 1123 format.
+   `t' specifies a source time.  If it is negative, the current time is specified.
+   `jl' specifies the jet lag of a location in seconds.
+   The return value is the string of the date in RFC 1123 format (Wdy, DD-Mon-YYYY hh:mm:dd TZD).
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call if it is no longer in use. */
+char *cbdatestrhttp(time_t t, int jl);
+
+
+/* Get the time value of a date string in decimal, hexadecimal, W3CDTF, or RFC 822 (1123).
+   `str' specifies a date string in decimal, hexadecimal, W3CDTF, or RFC 822 (1123).
+   The return value is the time value of the date or -1 if the format is invalid. */
+time_t cbstrmktime(const char *str);
+
+
 /* Get user and system processing times.
    `usrp' specifies the pointer to a variable to which the user processing time is assigned.
    If it is `NULL', it is not used.  The unit of time is seconds.
@@ -903,10 +1067,51 @@ void cbstdiobin(void);
 
 
 /*************************************************************************************************
- * Features for Experts
+ * features for experts
  *************************************************************************************************/
 
 
+/* Show error message on the standard error output and exit.
+   `message' specifies an error message.
+   This function does not return. */
+void *cbmyfatal(const char *message);
+
+
+/* Add an allocated element at the end of a list.
+   `list' specifies a list handle.
+   `ptr' specifies the pointer to the region of an element.  The region should be allocated with
+   malloc and it is released by the function.
+   `size' specifies the size of the region. */
+void cblistpushbuf(CBLIST *list, char *ptr, int size);
+
+
+/* Get a map handle with specifying the number of buckets.
+   `bnum' specifies the number of buckets.
+   The return value is a map handle. */
+CBMAP *cbmapopenex(int bnum);
+
+
+/* Store a record with an allocated region.
+   `map' specifies a map handle.
+   `kbuf' specifies the pointer to the region of a key.
+   `ksiz' specifies the size of the region of the key.
+   `vbuf' specifies the pointer to the region of a value.  The region should be allocated with
+   malloc and it is released by the function.
+   `vsiz' specifies the size of the region of the value.
+   If the key overlaps, the existing record is overwritten. */
+void cbmapputvbuf(CBMAP *map, const char *kbuf, int ksiz, char *vbuf, int vsiz);
+
+
+/* Alias of `cbmalloc'. */
+#define CB_MALLOC(ptr, size) \
+  (((ptr) = malloc(size)) ? (ptr) : cbmyfatal("out of memory"))
+
+
+/* Alias of `cbrealloc'. */
+#define CB_REALLOC(ptr, size) \
+  (((ptr) = realloc((ptr), (size))) ? (ptr) : cbmyfatal("out of memory"))
+
+
 /* Alias of `cbdatumptr'. */
 #define CB_DATUMPTR(datum) ((const char *)(datum->dptr))
 
@@ -924,6 +1129,12 @@ void cbstdiobin(void);
 #define CB_LISTVAL(list,index,sp) ((const char *)(list->array[list->start+index].dptr))
 
 
+/* Alias of `cblistval'.
+   However, `sp' is needed. */
+#define CB_LISTVAL2(list,index,sp) (*sp = list->array[list->start+index].dsize,\
+  (const char *)(list->array[list->start+index].dptr))
+
+
 
 #if defined(__cplusplus)                 /* export for C++ */
 }
