@@ -102,7 +102,7 @@ void usage(void){
   fprintf(stderr, "  %s csv [-d] [-t] [-l] [-e expr] [-html] [file]\n", progname);
   fprintf(stderr, "  %s xml [-d] [-p] [-l] [-e expr] [-tsv] [file]\n", progname);
   if((tmp = cbdeflate("", 0, &tsiz)) != NULL){
-    fprintf(stderr, "  %s zlib [-d] [-crc] [file]\n", progname);
+    fprintf(stderr, "  %s zlib [-d] [-gz] [-crc] [file]\n", progname);
     free(tmp);
   }
   if((tmp = cbiconv("", 0, "US-ASCII", "US-ASCII", NULL, NULL)) != NULL){
@@ -675,15 +675,18 @@ int runxml(int argc, char **argv){
 /* parse arguments of zlib command */
 int runzlib(int argc, char **argv){
   unsigned int sum;
-  int i, bsiz, rsiz, dec, crc;
+  int i, bsiz, rsiz, dec, gz, crc;
   char *file, *buf, *res;
   dec = FALSE;
+  gz = FALSE;
   crc = FALSE;
   file = NULL;
   for(i = 2; i < argc; i++){
     if(!file && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-d")){
         dec = TRUE;
+      } else if(!strcmp(argv[i], "-gz")){
+        gz = TRUE;
       } else if(!strcmp(argv[i], "-crc")){
         crc = TRUE;
       } else {
@@ -712,7 +715,7 @@ int runzlib(int argc, char **argv){
     }
     putchar('\n');
   } else if(dec){
-    if(!(res = cbinflate(buf, bsiz, &rsiz))){
+    if(!(res = gz ? cbgzdecode(buf, bsiz, &rsiz) : cbinflate(buf, bsiz, &rsiz))){
       fprintf(stderr, "%s: inflate failed\n", progname);
       free(buf);
       return 1;
@@ -722,7 +725,7 @@ int runzlib(int argc, char **argv){
     }
     free(res);
   } else {
-    if(!(res = cbdeflate(buf, bsiz, &rsiz))){
+    if(!(res = gz ? cbgzencode(buf, bsiz, &rsiz) : cbdeflate(buf, bsiz, &rsiz))){
       fprintf(stderr, "%s: deflate failed\n", progname);
       free(buf);
       return 1;
