@@ -29,6 +29,7 @@
 #define FALSE          0                 /* boolean false */
 
 #define RECBUFSIZ      32                /* buffer for records */
+#define TEXTBUFSIZ     262144            /* buffer for text */
 
 
 /* for RISC OS */
@@ -46,6 +47,7 @@ const char *progname;                    /* program name */
 int main(int argc, char **argv);
 void usage(void);
 int runsort(int argc, char **argv);
+int runstrstr(int argc, char **argv);
 int runlist(int argc, char **argv);
 int runmap(int argc, char **argv);
 int runwicked(int argc, char **argv);
@@ -54,6 +56,7 @@ int printfflush(const char *format, ...);
 int strpcmp(const void *ap, const void *bp);
 int myrand(void);
 int dosort(int rnum, int disp);
+int dostrstr(int rnum, int disp);
 int dolist(int rnum, int disp);
 int domap(int rnum, int bnum, int disp);
 int dowicked(int rnum);
@@ -69,6 +72,8 @@ int main(int argc, char **argv){
   rv = 0;
   if(!strcmp(argv[1], "sort")){
     rv = runsort(argc, argv);
+  } else if(!strcmp(argv[1], "strstr")){
+    rv = runstrstr(argc, argv);
   } else if(!strcmp(argv[1], "list")){
     rv = runlist(argc, argv);
   } else if(!strcmp(argv[1], "map")){
@@ -90,6 +95,7 @@ void usage(void){
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s sort [-d] rnum\n", progname);
+  fprintf(stderr, "  %s strstr [-d] rnum\n", progname);
   fprintf(stderr, "  %s list [-d] rnum\n", progname);
   fprintf(stderr, "  %s map [-d] rnum [bnum]\n", progname);
   fprintf(stderr, "  %s wicked rnum\n", progname);
@@ -127,6 +133,34 @@ int runsort(int argc, char **argv){
 }
 
 
+/* parse arguments of strstr command */
+int runstrstr(int argc, char **argv){
+  int i, rnum, disp, rv;
+  char *rstr;
+  rstr = NULL;
+  rnum = 0;
+  disp = FALSE;
+  for(i = 2; i < argc; i++){
+    if(argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-d")){
+        disp = TRUE;
+      } else {
+        usage();
+      }
+    } else if(!rstr){
+      rstr = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!rstr) usage();
+  rnum = atoi(rstr);
+  if(rnum < 1) usage();
+  rv = dostrstr(rnum, disp);
+  return rv;
+}
+
+
 /* parse arguments of list command */
 int runlist(int argc, char **argv){
   int i, rnum, disp, rv;
@@ -303,6 +337,42 @@ int dosort(int rnum, int disp){
 }
 
 
+/* perform strstr command */
+int dostrstr(int rnum, int disp){
+  char *text, buf[RECBUFSIZ], *std, *kmp, *bm;
+  int i, j, len, err;
+  text = cbmalloc(TEXTBUFSIZ);
+  for(i = 0; i < TEXTBUFSIZ - 1; i++){
+    text[i] = 'a' + myrand() % ('z' - 'a');
+  }
+  text[i] = '\0';
+  err = FALSE;
+  if(!disp) printfflush("Locating substrings ... ");
+  for(i = 0; i < rnum; i++){
+    len = myrand() % (RECBUFSIZ - 1);
+    for(j = 0; j < len; j++){
+      buf[j] = 'a' + myrand() % ('z' - 'a');
+    }
+    buf[j] = 0;
+    std = strstr(text, buf);
+    kmp = cbstrstrkmp(text, buf);
+    bm = cbstrstrbm(text, buf);
+    if(kmp != std || bm != std){
+      err = TRUE;
+      break;
+    }
+    if(disp && std) printf("%s\n", buf);
+  }
+  if(err) fprintf(stderr, "%s: sorting failed\n", progname);
+  if(!disp && !err){
+    printfflush("ok\n");
+    printfflush("all ok\n\n");
+  }
+  free(text);
+  return err ? 1 : 0;
+}
+
+
 /* perform list command */
 int dolist(int rnum, int disp){
   CBLIST *list;
