@@ -36,7 +36,7 @@
 #define DEFENC      "US-ASCII"           /* default encoding */
 #define DEFLANG     "en"                 /* default language */
 #define DEFTITLE    "QDBM on WWW"        /* default title */
-#define DEFDBDIR    "qadmdb"             /* directory containing database files */
+#define DEFDATADIR  "qadmdir"            /* directory containing database files */
 #define RDATAMAX    262144               /* max size of data to read */
 #define DBNAMEMAX   128                  /* max size of the name of a database */
 #define PATHBUFSIZ  1024                 /* size of a buffer for path string */
@@ -105,16 +105,17 @@ void villareclist(const char *dbname);
 int main(int argc, char **argv){
   CBMAP *params;
   CBLIST *lines;
-  const char *tmp, *dbdir, *dbname, *key, *val;
+  const char *tmp, *datadir, *dbname, *key, *val;
   char *pinfo, *tkey;
   int i, act, sure, dbt;
   /* set configurations */
+  cbstdiobin();
   scriptname = argv[0];
   if((tmp = getenv("SCRIPT_NAME")) != NULL) scriptname = tmp;
   enc = NULL;
   lang = NULL;
   title = NULL;
-  dbdir = NULL;
+  datadir = NULL;
   keychop = FALSE;
   valchop = FALSE;
   if((lines = cbreadlines(CONFFILE)) != NULL){
@@ -126,8 +127,8 @@ int main(int argc, char **argv){
         lang = skiplabel(tmp);
       } else if(cbstrfwmatch(tmp, "title:")){
         title = skiplabel(tmp);
-      } else if(cbstrfwmatch(tmp, "dbdir:")){
-        dbdir = skiplabel(tmp);
+      } else if(cbstrfwmatch(tmp, "datadir:")){
+        datadir = skiplabel(tmp);
       } else if(cbstrfwmatch(tmp, "keychop:")){
         if(!strcmp(skiplabel(tmp), "true")) keychop = TRUE;
       } else if(cbstrfwmatch(tmp, "valchop:")){
@@ -138,7 +139,7 @@ int main(int argc, char **argv){
   if(!enc) enc = DEFENC;
   if(!lang) lang = DEFLANG;
   if(!title) title = DEFTITLE;
-  if(!dbdir) dbdir = DEFDBDIR;
+  if(!datadir) datadir = DEFDATADIR;
   /* read parameters */
   dbname = NULL;
   act = ACTNONE;
@@ -156,7 +157,7 @@ int main(int argc, char **argv){
   if(dbname && strlen(dbname) > DBNAMEMAX) dbname = NULL;
   pinfo = getpathinfo();
   /* show page or send data */
-  if(chdir(dbdir) == -1){
+  if(chdir(datadir) == -1){
     senderror(500, "Internal Server Error", "Could not change the current directory.");
   } else if(act == ACTDBDOWN){
     dodbdown(dbname);
