@@ -71,7 +71,7 @@ int doinform(const char *name);
 int doremove(const char *name);
 int dorepair(const char *name);
 int doexportdb(const char *name, const char *dir);
-int doimportdb(const char *name, const char *dir);
+int doimportdb(const char *name, const char *dir, int bnum, int dnum);
 
 
 /* main routine */
@@ -134,7 +134,7 @@ void usage(void){
   fprintf(stderr, "  %s remove name\n", progname);
   fprintf(stderr, "  %s repair name\n", progname);
   fprintf(stderr, "  %s exportdb name dir\n", progname);
-  fprintf(stderr, "  %s importdb name dir\n", progname);
+  fprintf(stderr, "  %s importdb [-bnum num] [-dnum num] name dir\n", progname);
   fprintf(stderr, "  %s version\n", progname);
   fprintf(stderr, "\n");
   exit(1);
@@ -542,12 +542,22 @@ int runexportdb(int argc, char **argv){
 /* parse arguments of importdb command */
 int runimportdb(int argc, char **argv){
   char *name, *dir;
-  int i, rv;
+  int i, bnum, dnum, rv;
   name = NULL;
   dir = NULL;
+  bnum = -1;
+  dnum = -1;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      usage();
+      if(!strcmp(argv[i], "-bnum")){
+        if(++i >= argc) usage();
+        bnum = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-dnum")){
+        if(++i >= argc) usage();
+        dnum = atoi(argv[i]);
+      } else {
+        usage();
+      }
     } else if(!name){
       name = argv[i];
     } else if(!dir){
@@ -557,7 +567,7 @@ int runimportdb(int argc, char **argv){
     }
   }
   if(!name || !dir) usage();
-  rv = doimportdb(name, dir);
+  rv = doimportdb(name, dir, bnum, dnum);
   return rv;
 }
 
@@ -829,9 +839,9 @@ int doexportdb(const char *name, const char *dir){
 
 
 /* perform importdb command */
-int doimportdb(const char *name, const char *dir){
+int doimportdb(const char *name, const char *dir, int bnum, int dnum){
   CURIA *curia;
-  if(!(curia = cropen(name, CR_OWRITER | CR_OCREAT | CR_OTRUNC, -1, -1))){
+  if(!(curia = cropen(name, CR_OWRITER | CR_OCREAT | CR_OTRUNC, bnum, dnum))){
     pdperror(name);
     return 1;
   }
