@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Curia
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -30,6 +30,8 @@
 #define CR_TMPFSUF     MYEXTSTR "crtmp"  /* suffix of a temporary directory */
 #define CR_LOBDIR      "lob"             /* name of the directory of large objects */
 #define CR_LOBDDEPTH   2                 /* depth of the directories of large objects */
+#define CR_NUMBUFSIZ   32                /* size of a buffer for a number */
+#define CR_IOBUFSIZ    8192              /* size of an I/O buffer */
 
 
 /* private function prototypes */
@@ -68,8 +70,10 @@ CURIA *cropen(const char *name, int omode, int bnum, int dnum){
     dpomode = DP_OWRITER;
     if(omode & CR_OCREAT) dpomode |= DP_OCREAT;
     if(omode & CR_OTRUNC) dpomode |= DP_OTRUNC;
+    if(omode & CR_OSPARSE) dpomode |= DP_OSPARSE;
   }
   if(omode & CR_ONOLCK) dpomode |= DP_ONOLCK;
+  if(omode & CR_OLCKNB) dpomode |= DP_OLCKNB;
   attr = NULL;
   lrnum = 0;
   if((omode & CR_OWRITER) && (omode & CR_OCREAT)){
@@ -86,9 +90,18 @@ CURIA *cropen(const char *name, int omode, int bnum, int dnum){
     }
     sprintf(path, "%s%c%s", name, MYPATHCHR, CR_DPNAME);
     if(!(attr = dpopen(path, dpomode, CR_ATTRBNUM))) return NULL;
-    if(!dpput(attr, CR_KEYDNUM, -1, (char *)&dnum, sizeof(int), DP_DOVER)){
-      dpclose(attr);
-      return NULL;
+    if(dprnum(attr) > 0){
+      if((dnum = crdpgetnum(attr, CR_KEYDNUM, -1)) < 1 ||
+         (lrnum = crdpgetnum(attr, CR_KEYLRNUM, -1)) < 0){
+        dpclose(attr);
+        dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+        return NULL;
+      }
+    } else {
+      if(!dpput(attr, CR_KEYDNUM, -1, (char *)&dnum, sizeof(int), DP_DOVER)){
+        dpclose(attr);
+        return NULL;
+      }
     }
   }
   if(!attr){
@@ -451,6 +464,13 @@ int crinode(CURIA *curia){
 }
 
 
+/* Get the last modified time of a database. */
+int crmtime(CURIA *curia){
+  assert(curia);
+  return dpmtime(curia->attr);
+}
+
+
 /* Remove a database directory. */
 int crremove(const char *name){
   struct stat sbuf;
@@ -543,6 +563,108 @@ int crrepair(const char *name){
 }
 
 
+/* Dump all records as endian independent data. */
+int crexportdb(CURIA *curia, const char *name){
+  char path[CR_PATHBUFSIZ], *kbuf, *vbuf, *pbuf;
+  int i, err, *fds, ksiz, vsiz, psiz;
+  assert(curia && name);
+  if(!(criterinit(curia))) return FALSE;
+  if(mkdir(name, CR_DIRMODE) == -1 && errno != EEXIST){
+    dpecodeset(DP_EMKDIR, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  fds = malloc(sizeof(int) * curia->dnum);
+  for(i = 0; i < curia->dnum; i++){
+    sprintf(path, "%s%c%04d", name, MYPATHCHR, i + 1);
+    if((fds[i] = open(path, O_RDWR | O_CREAT | O_TRUNC, CR_FILEMODE)) == -1){
+      if(!err) dpecodeset(DP_EOPEN, __FILE__, __LINE__);
+      err = TRUE;
+      break;
+    }
+  }
+  while(!err && (kbuf = criternext(curia, &ksiz)) != NULL){
+    if((vbuf = crget(curia, kbuf, ksiz, 0, -1, &vsiz)) != NULL){
+      if((pbuf = malloc(ksiz + vsiz + CR_NUMBUFSIZ * 2)) != NULL){
+        psiz = 0;
+        psiz += sprintf(pbuf + psiz, "%X\n%X\n", ksiz, vsiz);
+        memcpy(pbuf + psiz, kbuf, ksiz);
+        psiz += ksiz;
+        pbuf[psiz++] = '\n';
+        memcpy(pbuf + psiz, vbuf, vsiz);
+        psiz += vsiz;
+        pbuf[psiz++] = '\n';
+        if(!crwrite(fds[curia->inum], pbuf, psiz)){
+          dpecodeset(DP_EWRITE, __FILE__, __LINE__);
+          err = TRUE;
+        }
+        free(pbuf);
+      } else {
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        err = TRUE;
+      }
+      free(vbuf);
+    } else {
+      err = TRUE;
+    }
+    free(kbuf);
+  }
+  for(i = 0; i < curia->dnum; i++){
+    if(fds[i] != -1 && close(fds[i]) == -1){
+      if(!err) dpecodeset(DP_ECLOSE, __FILE__, __LINE__);
+      err = TRUE;
+    }
+  }
+  free(fds);
+  return !err && !crfatalerror(curia);
+}
+
+
+/* Load all records from endian independent data. */
+int crimportdb(CURIA *curia, const char *name){
+  DEPOT *depot;
+  char ipath[CR_PATHBUFSIZ], opath[CR_PATHBUFSIZ], *kbuf, *vbuf;
+  int i, err, ksiz, vsiz;
+  struct stat sbuf;
+  assert(curia && name);
+  if(!curia->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(crrnum(curia) > 0){
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  for(i = 0; !err && i < CR_DPMAX; i++){
+    sprintf(ipath, "%s%c%04d", name, MYPATHCHR, i + 1);
+    if(stat(ipath, &sbuf) == -1 || !S_ISREG(sbuf.st_mode)) break;
+    sprintf(opath, "%s%c%04d%s", curia->name, MYPATHCHR, i + 1, CR_TMPFSUF);
+    if((depot = dpopen(opath, DP_OWRITER | DP_OCREAT | DP_OTRUNC, -1)) != NULL){
+      if(dpimportdb(depot, ipath)){
+        dpiterinit(depot);
+        while((kbuf = dpiternext(depot, &ksiz)) != NULL){
+          if((vbuf = dpget(depot, kbuf, ksiz, 0, -1, &vsiz)) != NULL){
+            if(!crput(curia, kbuf, ksiz, vbuf, vsiz, CR_DKEEP)) err = TRUE;
+            free(vbuf);
+          } else {
+            err = TRUE;
+          }
+          free(kbuf);
+        }
+      } else {
+        err = TRUE;
+      }
+      if(!dpclose(depot)) err = TRUE;
+      if(!dpremove(opath)) err = TRUE;
+    } else {
+      err = TRUE;
+    }
+  }
+  return !err && !crfatalerror(curia);
+}
+
+
 /* Store a large object. */
 int crputlob(CURIA *curia, const char *kbuf, int ksiz, const char *vbuf, int vsiz, int dmode){
   char *path;
@@ -704,7 +826,7 @@ int crrnumlob(CURIA *curia){
 
 
 /*************************************************************************************************
- * Features for Experts
+ * features for experts
  *************************************************************************************************/
 
 
