@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Depot
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -21,6 +21,7 @@
 #define DP_MAGICNUMB   "[DEPOT]\n\f"     /* magic number on environments of big endian */
 #define DP_MAGICNUML   "[depot]\n\f"     /* magic number on environments of little endian */
 #define DP_HEADSIZ     48                /* size of the reagion of the header */
+#define DP_LIBVEROFF   12                /* offset of the region for flags */
 #define DP_FLAGSOFF    16                /* offset of the region for flags */
 #define DP_FSIZOFF     24                /* offset of the region for the file size */
 #define DP_BNUMOFF     32                /* offset of the region for the bucket number */
@@ -33,6 +34,7 @@
 #define DP_TMPFSUF     MYEXTSTR "dptmp"  /* suffix of a temporary file */
 #define DP_OPTBLOAD    0.25              /* ratio of bucket loading at optimization */
 #define DP_OPTRUNIT    256               /* number of records in a process of optimization */
+#define DP_NUMBUFSIZ   32                /* size of a buffer for a number */
 #define DP_IOBUFSIZ    8192              /* size of an I/O buffer */
 
 enum {                                   /* enumeration for a record header */
@@ -55,7 +57,7 @@ enum {                                   /* enumeration for the flag of a record
 /* private function prototypes */
 static int dpbigendian(void);
 static char *dpstrdup(const char *str);
-static int dplock(int fd, int ex);
+static int dplock(int fd, int ex, int nb);
 static int dpwrite(int fd, const void *buf, int size);
 static int dpseekwrite(int fd, int off, const void *buf, int size);
 static int dpseekwritenum(int fd, int off, int num);
@@ -124,8 +126,8 @@ const char *dperrmsg(int ecode){
 
 /* Get a database handle. */
 DEPOT *dpopen(const char *name, int omode, int bnum){
-  int mode, fd, inode, fsiz, rnum, c, msiz;
-  char hbuf[DP_HEADSIZ], *map;
+  int mode, fd, inode, mtime, fsiz, rnum, msiz;
+  char hbuf[DP_HEADSIZ], *map, c;
   struct stat sbuf;
   DEPOT *depot;
   assert(name);
@@ -139,7 +141,7 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
     return NULL;
   }
   if(!(omode & DP_ONOLCK)){
-    if(!dplock(fd, omode & DP_OWRITER)){
+    if(!dplock(fd, omode & DP_OWRITER, omode & DP_OLCKNB)){
       close(fd);
       return NULL;
     }
@@ -157,6 +159,7 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
     return NULL;
   }
   inode = sbuf.st_ino;
+  mtime = sbuf.st_mtime;
   fsiz = sbuf.st_size;
   if((omode & DP_OWRITER) && fsiz == 0){
     memset(hbuf, 0, DP_HEADSIZ);
@@ -165,19 +168,38 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
     } else {
       memcpy(hbuf, DP_MAGICNUML, strlen(DP_MAGICNUML));
     }
+    sprintf(hbuf + DP_LIBVEROFF, "%d", _QDBM_LIBVER / 100);
     bnum = bnum < 1 ? DP_DEFBNUM : bnum;
     bnum = dpgetprime(bnum);
     memcpy(hbuf + DP_BNUMOFF, &bnum, sizeof(int));
     rnum = 0;
     memcpy(hbuf + DP_RNUMOFF, &rnum, sizeof(int));
-    c = 0;
     fsiz = DP_HEADSIZ + bnum * sizeof(int);
     memcpy(hbuf + DP_FSIZOFF, &fsiz, sizeof(int));
-    if(!dpseekwrite(fd, 0, hbuf, DP_HEADSIZ) ||
-       !dpseekwrite(fd, fsiz - 1, &c, 1)){
+    if(!dpseekwrite(fd, 0, hbuf, DP_HEADSIZ)){
       close(fd);
       return NULL;
     }
+    if(omode & DP_OSPARSE){
+      c = 0;
+      if(!dpseekwrite(fd, fsiz - 1, &c, 1)){
+        close(fd);
+        return NULL;
+      }
+    } else {
+      if(!(map = malloc(bnum * sizeof(int)))){
+        close(fd);
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        return NULL;
+      }
+      memset(map, 0, bnum * sizeof(int));
+      if(!dpseekwrite(fd, DP_HEADSIZ, map, bnum * sizeof(int))){
+        free(map);
+        close(fd);
+        return NULL;
+      }
+      free(map);
+    }
   }
   if(!dpseekread(fd, 0, hbuf, DP_HEADSIZ)){
     close(fd);
@@ -214,6 +236,7 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
   }
   depot->wmode = (mode & DP_OWRITER);
   depot->inode = inode;
+  depot->mtime = mtime;
   depot->fd = fd;
   depot->fsiz = fsiz;
   depot->map = map;
@@ -891,6 +914,13 @@ int dpinode(DEPOT *depot){
 }
 
 
+/* Get the last modified time of a database. */
+int dpmtime(DEPOT *depot){
+  assert(depot);
+  return depot->mtime;
+}
+
+
 /* Get the file descriptor of a database file. */
 int dpfdesc(DEPOT *depot){
   assert(depot);
@@ -1005,6 +1035,132 @@ int dprepair(const char *name){
 }
 
 
+/* Dump all records as endian independent data. */
+int dpexportdb(DEPOT *depot, const char *name){
+  char *kbuf, *vbuf, *pbuf;
+  int fd, err, ksiz, vsiz, psiz;
+  assert(depot && name);
+  if(!(dpiterinit(depot))) return FALSE;
+  if((fd = open(name, O_RDWR | O_CREAT | O_TRUNC, DP_FILEMODE)) == -1){
+    dpecodeset(DP_EOPEN, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  while(!err && (kbuf = dpiternext(depot, &ksiz)) != NULL){
+    if((vbuf = dpget(depot, kbuf, ksiz, 0, -1, &vsiz)) != NULL){
+      if((pbuf = malloc(ksiz + vsiz + DP_NUMBUFSIZ * 2)) != NULL){
+        psiz = 0;
+        psiz += sprintf(pbuf + psiz, "%X\n%X\n", ksiz, vsiz);
+        memcpy(pbuf + psiz, kbuf, ksiz);
+        psiz += ksiz;
+        pbuf[psiz++] = '\n';
+        memcpy(pbuf + psiz, vbuf, vsiz);
+        psiz += vsiz;
+        pbuf[psiz++] = '\n';
+        if(!dpwrite(fd, pbuf, psiz)){
+          dpecodeset(DP_EWRITE, __FILE__, __LINE__);
+          err = TRUE;
+        }
+        free(pbuf);
+      } else {
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        err = TRUE;
+      }
+      free(vbuf);
+    } else {
+      err = TRUE;
+    }
+    free(kbuf);
+  }
+  if(close(fd) == -1){
+    if(!err) dpecodeset(DP_ECLOSE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  return !err && !dpfatalerror(depot);
+}
+
+
+/* Load all records from endian independent data. */
+int dpimportdb(DEPOT *depot, const char *name){
+  char mbuf[DP_IOBUFSIZ], *rbuf;
+  int i, j, fd, err, fsiz, off, msiz, ksiz, vsiz, hlen;
+  struct stat sbuf;
+  assert(depot && name);
+  if(!depot->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(dprnum(depot) > 0){
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if((fd = open(name, O_RDONLY, DP_FILEMODE)) == -1){
+    dpecodeset(DP_EOPEN, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(fstat(fd, &sbuf) == -1 || !S_ISREG(sbuf.st_mode)){
+    dpecodeset(DP_ESTAT, __FILE__, __LINE__);
+    close(fd);
+    return FALSE;
+  }
+  err = FALSE;
+  fsiz = sbuf.st_size;
+  off = 0;
+  while(!err && off < fsiz){
+    msiz = fsiz - off;
+    if(msiz > DP_IOBUFSIZ) msiz = DP_IOBUFSIZ;
+    if(!dpseekread(fd, off, mbuf, msiz)){
+      err = TRUE;
+      break;
+    }
+    hlen = 0;
+    ksiz = -1;
+    vsiz = -1;
+    for(i = 0; i < msiz; i++){
+      if(mbuf[i] == '\n'){
+        mbuf[i] = '\0';
+        ksiz = strtol(mbuf, NULL, 16);
+        for(j = i + 1; j < msiz; j++){
+          if(mbuf[j] == '\n'){
+            mbuf[j] = '\0';
+            vsiz = strtol(mbuf + i + 1, NULL, 16);
+            hlen = j + 1;
+            break;
+          }
+        }
+        break;
+      }
+    }
+    if(ksiz < 0 || vsiz < 0 || hlen < 4){
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      err = TRUE;
+      break;
+    }
+    if(hlen + ksiz + vsiz + 2 < DP_IOBUFSIZ){
+      if(!dpput(depot, mbuf + hlen, ksiz, mbuf + hlen + ksiz + 1, vsiz, DP_DKEEP)) err = TRUE;
+    } else {
+      if((rbuf = malloc(ksiz + vsiz + 2)) != NULL){
+        if(dpseekread(fd, off + hlen, rbuf, ksiz + vsiz + 2)){
+          if(!dpput(depot, rbuf, ksiz, rbuf + ksiz + 1, vsiz, DP_DKEEP)) err = TRUE;
+        } else {
+          err = TRUE;
+        }
+        free(rbuf);
+      } else {
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        err = TRUE;
+      }
+    }
+    off += hlen + ksiz + vsiz + 2;
+  }
+  if(close(fd) == -1){
+    if(!err) dpecodeset(DP_ECLOSE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  return !err && !dpfatalerror(depot);
+}
+
+
 /* Hash function used inside Depot. */
 int dpinnerhash(const char *kbuf, int ksiz){
   assert(kbuf);
@@ -1030,7 +1186,7 @@ int dpprimenum(int num){
 
 
 /*************************************************************************************************
- * Features for Experts
+ * features for experts
  *************************************************************************************************/
 
 
@@ -1042,6 +1198,10 @@ char *dpsysname = _QDBM_SYSNAME;
 int dpdbgfd = -1;
 
 
+/* Whether this build is reentrant. */
+const int dpisreentrant = _qdbm_ptsafe;
+
+
 /* Set the last happened error code. */
 void dpecodeset(int ecode, const char *file, int line){
   char iobuf[DP_IOBUFSIZ];
@@ -1084,7 +1244,7 @@ int dpmemsync(DEPOT *depot){
   }
   *((int *)(depot->map + DP_FSIZOFF)) = depot->fsiz;
   *((int *)(depot->map + DP_RNUMOFF)) = depot->rnum;
-  if(msync(depot->map, depot->msiz, MS_ASYNC) == -1){
+  if(msync(depot->map, depot->msiz, MS_SYNC) == -1){
     dpecodeset(DP_EMAP, __FILE__, __LINE__);
     depot->fatal = TRUE;
     return FALSE;
@@ -1144,8 +1304,9 @@ static char *dpstrdup(const char *str){
 /* Lock a file descriptor.
    `fd' specifies a file descriptor.
    `ex' specifies whether an exclusive lock or a shared lock is performed.
+   `nb' specifies whether to request with non-blocking.
    The return value is true if successful, else, it is false. */
-static int dplock(int fd, int ex){
+static int dplock(int fd, int ex, int nb){
   struct flock lock;
   assert(fd >= 0);
   memset(&lock, 0, sizeof(struct flock));
@@ -1154,7 +1315,7 @@ static int dplock(int fd, int ex){
   lock.l_start = 0;
   lock.l_len = 0;
   lock.l_pid = 0;
-  while(fcntl(fd, F_SETLKW, &lock) == -1){
+  while(fcntl(fd, nb ? F_SETLK : F_SETLKW, &lock) == -1){
     if(errno != EINTR){
       dpecodeset(DP_ELOCK, __FILE__, __LINE__);
       return FALSE;
@@ -1663,8 +1824,8 @@ static int dprecrewrite(DEPOT *depot, int off, int rsiz, const char *kbuf, int k
    The return value is the offset of the record, or, -1 on failure. */
 static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
                        int hash, int left, int right){
-  char ebuf[DP_WRTBUFSIZ];
-  int head[DP_RHNUM], asiz, psiz, off, hoff, koff, voff, c;
+  char ebuf[DP_WRTBUFSIZ], *hbuf;
+  int head[DP_RHNUM], asiz, psiz, off;
   assert(depot && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
   psiz = dppadsize(depot, vsiz);
   head[DP_RHIFLAGS] = 0;
@@ -1683,14 +1844,19 @@ static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbu
     memset(ebuf + sizeof(head) + ksiz + vsiz, 0, psiz);
     if(!dpseekwrite(depot->fd, off, ebuf, asiz)) return -1;
   } else {
-    hoff = off;
-    koff = hoff + sizeof(head);
-    voff = koff + ksiz;
-    c = 0;
-    if(!dpseekwrite(depot->fd, hoff, head, sizeof(head)) ||
-       !dpseekwrite(depot->fd, koff, kbuf, ksiz) ||
-       !dpseekwrite(depot->fd, voff, vbuf, vsiz) ||
-       (psiz > 0 && !dpseekwrite(depot->fd, voff + vsiz + psiz - 1, &c, 1))) return -1;
+    if(!(hbuf = malloc(asiz))){
+      dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+      return -1;
+    }
+    memcpy(hbuf, head, sizeof(head));
+    memcpy(hbuf + sizeof(head), kbuf, ksiz);
+    memcpy(hbuf + sizeof(head) + ksiz, vbuf, vsiz);
+    memset(hbuf + sizeof(head) + ksiz + vsiz, 0, psiz);
+    if(!dpseekwrite(depot->fd, off, hbuf, asiz)){
+      free(hbuf);
+      return -1;
+    }
+    free(hbuf);
   }
   depot->fsiz += asiz;
   return off;
