@@ -69,7 +69,7 @@ int doinform(const char *name);
 int doremove(const char *name);
 int dorepair(const char *name);
 int doexportdb(const char *name, const char *file);
-int doimportdb(const char *name, const char *file);
+int doimportdb(const char *name, const char *file, int bnum);
 
 
 /* main routine */
@@ -132,7 +132,7 @@ void usage(void){
   fprintf(stderr, "  %s remove name\n", progname);
   fprintf(stderr, "  %s repair name\n", progname);
   fprintf(stderr, "  %s exportdb name file\n", progname);
-  fprintf(stderr, "  %s importdb name file\n", progname);
+  fprintf(stderr, "  %s importdb [-bnum num] name file\n", progname);
   fprintf(stderr, "  %s version\n", progname);
   fprintf(stderr, "\n");
   exit(1);
@@ -527,12 +527,18 @@ int runexportdb(int argc, char **argv){
 /* parse arguments of importdb command */
 int runimportdb(int argc, char **argv){
   char *name, *file;
-  int i, rv;
+  int i, bnum, rv;
   name = NULL;
   file = NULL;
+  bnum = -1;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      usage();
+      if(!strcmp(argv[i], "-bnum")){
+        if(++i >= argc) usage();
+        bnum = atoi(argv[i]);
+      } else {
+        usage();
+      }
     } else if(!name){
       name = argv[i];
     } else if(!file){
@@ -542,7 +548,7 @@ int runimportdb(int argc, char **argv){
     }
   }
   if(!name || !file) usage();
-  rv = doimportdb(name, file);
+  rv = doimportdb(name, file, bnum);
   return rv;
 }
 
@@ -791,9 +797,9 @@ int doexportdb(const char *name, const char *file){
 
 
 /* perform importdb command */
-int doimportdb(const char *name, const char *file){
+int doimportdb(const char *name, const char *file, int bnum){
   DEPOT *depot;
-  if(!(depot = dpopen(name, DP_OWRITER | DP_OCREAT | DP_OTRUNC, -1))){
+  if(!(depot = dpopen(name, DP_OWRITER | DP_OCREAT | DP_OTRUNC, bnum))){
     pdperror(name);
     return 1;
   }
