@@ -54,9 +54,9 @@ void pdperror(const char *name);
 int myrand(void);
 int dowrite(const char *name, int rnum, int bnum, int sparse);
 int doread(const char *name, int wb);
-int dorcat(const char *name, int rnum, int bnum, int pnum, int align);
+int dorcat(const char *name, int rnum, int bnum, int pnum, int align, int cb);
 int docombo(const char *name);
-int dowicked(const char *name, int rnum);
+int dowicked(const char *name, int rnum, int cb);
 
 
 /* main routine */
@@ -92,9 +92,9 @@ void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s write [-s] name rnum bnum\n", progname);
   fprintf(stderr, "  %s read [-wb] name\n", progname);
-  fprintf(stderr, "  %s rcat name rnum bnum pnum align\n", progname);
+  fprintf(stderr, "  %s rcat [-c] name rnum bnum pnum align\n", progname);
   fprintf(stderr, "  %s combo name\n", progname);
-  fprintf(stderr, "  %s wicked name rnum\n", progname);
+  fprintf(stderr, "  %s wicked [-c] name rnum\n", progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -164,7 +164,7 @@ int runread(int argc, char **argv){
 /* parse arguments of rcat command */
 int runrcat(int argc, char **argv){
   char *name, *rstr, *bstr, *pstr, *astr;
-  int i, rnum, bnum, pnum, align, rv;
+  int i, rnum, bnum, pnum, align, cb, rv;
   name = NULL;
   rstr = NULL;
   bstr = NULL;
@@ -174,9 +174,14 @@ int runrcat(int argc, char **argv){
   bnum = 0;
   pnum = 0;
   align = 0;
+  cb = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      usage();
+      if(!strcmp(argv[i], "-c")){
+        cb = TRUE;
+      } else {
+        usage();
+      }
     } else if(!name){
       name = argv[i];
     } else if(!rstr){
@@ -197,7 +202,7 @@ int runrcat(int argc, char **argv){
   pnum = atoi(pstr);
   align = atoi(astr);
   if(rnum < 1 || bnum < 1 || pnum < 1) usage();
-  rv = dorcat(name, rnum, bnum, pnum, align);
+  rv = dorcat(name, rnum, bnum, pnum, align, cb);
   return rv;
 }
 
@@ -225,13 +230,18 @@ int runcombo(int argc, char **argv){
 /* parse arguments of wicked command */
 int runwicked(int argc, char **argv){
   char *name, *rstr;
-  int i, rnum, rv;
+  int i, rnum, cb, rv;
   name = NULL;
   rstr = NULL;
   rnum = 0;
+  cb = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      usage();
+      if(!strcmp(argv[i], "-c")){
+        cb = TRUE;
+      } else {
+        usage();
+      }
     } else if(!name){
       name = argv[i];
     } else if(!rstr){
@@ -243,7 +253,7 @@ int runwicked(int argc, char **argv){
   if(!name || !rstr) usage();
   rnum = atoi(rstr);
   if(rnum < 1) usage();
-  rv = dowicked(name, rnum);
+  rv = dowicked(name, rnum, cb);
   return rv;
 }
 
@@ -279,7 +289,7 @@ int dowrite(const char *name, int rnum, int bnum, int sparse){
   DEPOT *depot;
   int i, omode, err, len;
   char buf[RECBUFSIZ];
-  printfflush("<Writing Test>\n  name=%s  rnum=%d  bnum=%d\n\n", name, rnum, bnum);
+  printfflush("<Writing Test>\n  name=%s  rnum=%d  bnum=%d  s=%d\n\n", name, rnum, bnum, sparse);
   /* open a database */
   omode = DP_OWRITER | DP_OCREAT | DP_OTRUNC | (sparse ? DP_OSPARSE : 0);
   if(!(depot = dpopen(name, omode, bnum))){
@@ -367,12 +377,14 @@ int doread(const char *name, int wb){
 
 
 /* perform rcat command */
-int dorcat(const char *name, int rnum, int bnum, int pnum, int align){
+int dorcat(const char *name, int rnum, int bnum, int pnum, int align, int cb){
   DEPOT *depot;
-  int i, err, len;
-  char buf[RECBUFSIZ];
-  printfflush("<Random Writing Test>\n  name=%s  rnum=%d  bnum=%d  pnum=%d  align=%d\n\n",
-              name, rnum, bnum, pnum, align);
+  CBMAP *map;
+  int i, err, len, ksiz, vsiz, rsiz;
+  const char *kbuf, *vbuf;
+  char buf[RECBUFSIZ], *rbuf;
+  printfflush("<Random Writing Test>\n  name=%s  rnum=%d  bnum=%d  pnum=%d  align=%d  c=%d\n\n",
+              name, rnum, bnum, pnum, align, cb);
   if(!(depot = dpopen(name, DP_OWRITER | DP_OCREAT | DP_OTRUNC, bnum))){
     pdperror(name);
     return 1;
@@ -382,6 +394,8 @@ int dorcat(const char *name, int rnum, int bnum, int pnum, int align){
     dpclose(depot);
     return 1;
   }
+  map = NULL;
+  if(cb) map = cbmapopen();
   err = FALSE;
   for(i = 1; i <= rnum; i++){
     len = sprintf(buf, "%08d", myrand() % pnum + 1);
@@ -390,6 +404,7 @@ int dorcat(const char *name, int rnum, int bnum, int pnum, int align){
       err = TRUE;
       break;
     }
+    if(map) cbmapputcat(map, buf, len, buf, len);
     if(rnum > 250 && i % (rnum / 250) == 0){
       putchar('.');
       fflush(stdout);
@@ -398,6 +413,27 @@ int dorcat(const char *name, int rnum, int bnum, int pnum, int align){
       }
     }
   }
+  if(map){
+    printfflush("Matching records ... ");
+    cbmapiterinit(map);
+    while((kbuf = cbmapiternext(map, &ksiz)) != NULL){
+      vbuf = cbmapget(map, kbuf, ksiz, &vsiz);
+      if(!(rbuf = dpget(depot, kbuf, ksiz, 0, -1, &rsiz))){
+        pdperror(name);
+        err = TRUE;
+        break;
+      }
+      if(rsiz != vsiz || memcmp(rbuf, vbuf, rsiz)){
+        fprintf(stderr, "%s: %s: unmatched record\n", progname, name);
+        free(rbuf);
+        err = TRUE;
+        break;
+      }
+      free(rbuf);
+    }
+    cbmapclose(map);
+    if(!err) printfflush("ok\n");
+  }
   if(!dpclose(depot)){
     pdperror(name);
     return 1;
@@ -620,9 +656,11 @@ int docombo(const char *name){
 
 
 /* perform wicked command */
-int dowicked(const char *name, int rnum){
+int dowicked(const char *name, int rnum, int cb){
   DEPOT *depot;
-  int i, len, err;
+  CBMAP *map;
+  int i, len, err, align, mksiz, mvsiz, rsiz;
+  const char *mkbuf, *mvbuf;
   char buf[RECBUFSIZ], vbuf[RECBUFSIZ], *val;
   printfflush("<Wicked Writing Test>\n  name=%s  rnum=%d\n\n", name, rnum);
   err = FALSE;
@@ -634,20 +672,25 @@ int dowicked(const char *name, int rnum){
     pdperror(name);
     err = TRUE;
   }
+  map = NULL;
+  if(cb) map = cbmapopen();
   for(i = 1; i <= rnum; i++){
     len = sprintf(buf, "%08d", myrand() % rnum + 1);
     switch(myrand() % 16){
     case 0:
       putchar('O');
       if(!dpput(depot, buf, len, buf, len, DP_DOVER)) err = TRUE;
+      if(map) cbmapput(map, buf, len, buf, len, TRUE);
       break;
     case 1:
       putchar('K');
       if(!dpput(depot, buf, len, buf, len, DP_DKEEP) && dpecode != DP_EKEEP) err = TRUE;
+      if(map) cbmapput(map, buf, len, buf, len, FALSE);
       break;
     case 2:
       putchar('D');
       if(!dpout(depot, buf, len) && dpecode != DP_ENOITEM) err = TRUE;
+      if(map) cbmapout(map, buf, len);
       break;
     case 3:
       putchar('G');
@@ -660,9 +703,18 @@ int dowicked(const char *name, int rnum){
     default:
       putchar('C');
       if(!dpput(depot, buf, len, buf, len, DP_DCAT)) err = TRUE;
+      if(map) cbmapputcat(map, buf, len, buf, len);
       break;
     }
     if(i % 50 == 0) printfflush(" (%08d)\n", i);
+    if(!err && rnum > 100 && myrand() % (rnum / 100) == 0){
+      if(myrand() % 10 == 0){
+        align = (myrand() % 4 + 1) * -1;
+      } else {
+        align = myrand() % 32;
+      }
+      if(!dpsetalign(depot, align)) err = TRUE;
+    }
     if(err){
       pdperror(name);
       break;
@@ -679,6 +731,7 @@ int dowicked(const char *name, int rnum){
       err = TRUE;
       break;
     }
+    if(map) cbmapputcat(map, buf, len, ":", -1);
     putchar(':');
     if(i % 50 == 0) printfflush(" (%08d)\n", i);
   }
@@ -711,6 +764,27 @@ int dowicked(const char *name, int rnum){
     putchar('@');
     if(i % 50 == 0) printfflush(" (%08d)\n", i);
   }
+  if(map){
+    printfflush("Matching records ... ");
+    cbmapiterinit(map);
+    while((mkbuf = cbmapiternext(map, &mksiz)) != NULL){
+      mvbuf = cbmapget(map, mkbuf, mksiz, &mvsiz);
+      if(!(val = dpget(depot, mkbuf, mksiz, 0, -1, &rsiz))){
+        pdperror(name);
+        err = TRUE;
+        break;
+      }
+      if(rsiz != mvsiz || memcmp(val, mvbuf, rsiz)){
+        fprintf(stderr, "%s: %s: unmatched record\n", progname, name);
+        free(val);
+        err = TRUE;
+        break;
+      }
+      free(val);
+    }
+    cbmapclose(map);
+    if(!err) printfflush("ok\n");
+  }
   if(!dpclose(depot)){
     pdperror(name);
     return 1;
