@@ -81,8 +81,10 @@ public class Curia implements ADBM {
   public static final int OTRUNC = 1 << 3;
   /** open mode: open without locking */
   public static final int ONOLCK = 1 << 4;
+  /** open mode: lock without blocking */
+  public static final int OLCKNB = 1 << 5;
   /** open mode: create as sparse files */
-  public static final int OSPARSE = 1 << 5;
+  public static final int OSPARSE = 1 << 6;
   //----------------------------------------------------------------
   // write modes
   //----------------------------------------------------------------
@@ -147,8 +149,9 @@ public class Curia implements ADBM {
    * `Curia.OCREAT', which means it creates a new database if not exist, `Curia.OTRUNC',
    * which means it creates a new database regardless if one exists.  Both of `Curia.OREADER'
    * and `Curia.OWRITER' can be added to by bitwise or: `Curia.ONOLCK', which means it opens a
-   * database directory without file locking.  `Curia.OCREAT' can be added to by bitwise or:
-   * `Curia.OSPARSE', which means it creates database files as sparse files.
+   * database directory without file locking, or `Curia.OLCKNB', which means locking is performed
+   * without blocking.  `Curia.OCREAT' can be added to by bitwise or: `Curia.OSPARSE', which
+   * means it creates database files as sparse files.
    * @param bnum the number of elements of the each bucket array.  If it is not more than 0,
    * the default value is specified.  The size of each bucket array is determined on creating,
    * and can not be changed except for by optimization of the database.  Suggested size of each
