@@ -351,20 +351,19 @@ struct dirent *_qdbm_win32_readdir(DIR *dir){
 #include <zlib.h>
 
 #define ZLIBBUFSIZ     8192
-#define ZLIBCLEVEL     5
 
 
-static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp);
-static char *_qdbm_inflate_impl(const char *ptr, int size, int *sp);
+static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp, int mode);
+static char *_qdbm_inflate_impl(const char *ptr, int size, int *sp, int mode);
 static unsigned int _qdbm_getcrc_impl(const char *ptr, int size);
 
 
-char *(*_qdbm_deflate)(const char *, int, int *) = _qdbm_deflate_impl;
-char *(*_qdbm_inflate)(const char *, int, int *) = _qdbm_inflate_impl;
+char *(*_qdbm_deflate)(const char *, int, int *, int) = _qdbm_deflate_impl;
+char *(*_qdbm_inflate)(const char *, int, int *, int) = _qdbm_inflate_impl;
 unsigned int (*_qdbm_getcrc)(const char *, int) = _qdbm_getcrc_impl;
 
 
-static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp){
+static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp, int mode){
   z_stream zs;
   char *buf, *swap;
   unsigned char obuf[ZLIBBUFSIZ];
@@ -373,7 +372,20 @@ static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp){
   zs.zalloc = Z_NULL;
   zs.zfree = Z_NULL;
   zs.opaque = Z_NULL;
-  if(deflateInit(&zs, ZLIBCLEVEL) != Z_OK) return NULL;
+  switch(mode){
+  case _QDBM_ZMRAW:
+    if(deflateInit2(&zs, 5, Z_DEFLATED, -15, 7, Z_DEFAULT_STRATEGY) != Z_OK)
+      return NULL;
+    break;
+  case _QDBM_ZMGZIP:
+    if(deflateInit2(&zs, 6, Z_DEFLATED, 15 + 16, 9, Z_DEFAULT_STRATEGY) != Z_OK)
+      return NULL;
+    break;
+  default:
+    if(deflateInit2(&zs, 6, Z_DEFLATED, 15, 8, Z_DEFAULT_STRATEGY) != Z_OK)
+      return NULL;
+    break;
+  }
   asiz = ZLIBBUFSIZ;
   if(!(buf = malloc(asiz))){
     deflateEnd(&zs);
@@ -406,7 +418,7 @@ static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp){
     return NULL;
   }
   osiz = ZLIBBUFSIZ - zs.avail_out;
-  if(bsiz + osiz > asiz){
+  if(bsiz + osiz + 1 > asiz){
     asiz = asiz * 2 + osiz;
     if(!(swap = realloc(buf, asiz))){
       free(buf);
@@ -417,13 +429,14 @@ static char *_qdbm_deflate_impl(const char *ptr, int size, int *sp){
   }
   memcpy(buf + bsiz, obuf, osiz);
   bsiz += osiz;
+  if(mode == _QDBM_ZMRAW) buf[bsiz++] = '\0';
   *sp = bsiz;
   deflateEnd(&zs);
   return buf;
 }
 
 
-static char *_qdbm_inflate_impl(const char *ptr, int size, int *sp){
+static char *_qdbm_inflate_impl(const char *ptr, int size, int *sp, int mode){
   z_stream zs;
   char *buf, *swap;
   unsigned char obuf[ZLIBBUFSIZ];
@@ -431,7 +444,17 @@ static char *_qdbm_inflate_impl(const char *ptr, int size, int *sp){
   zs.zalloc = Z_NULL;
   zs.zfree = Z_NULL;
   zs.opaque = Z_NULL;
-  if(inflateInit(&zs) != Z_OK) return NULL;
+  switch(mode){
+  case _QDBM_ZMRAW:
+    if(inflateInit2(&zs, -15) != Z_OK) return NULL;
+    break;
+  case _QDBM_ZMGZIP:
+    if(inflateInit2(&zs, 15 + 16) != Z_OK) return NULL;
+    break;
+  default:
+    if(inflateInit2(&zs, 15) != Z_OK) return NULL;
+    break;
+  }
   asiz = ZLIBBUFSIZ;
   if(!(buf = malloc(asiz))){
     deflateEnd(&zs);
@@ -493,8 +516,8 @@ static unsigned int _qdbm_getcrc_impl(const char *ptr, int size){
 #else
 
 
-char *(*_qdbm_deflate)(const char *, int, int *) = NULL;
-char *(*_qdbm_inflate)(const char *, int, int *) = NULL;
+char *(*_qdbm_deflate)(const char *, int, int *, int) = NULL;
+char *(*_qdbm_inflate)(const char *, int, int *, int) = NULL;
 unsigned int (*_qdbm_getcrc)(const char *, int) = NULL;
 
 
