@@ -210,7 +210,7 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
   tmp = cbmapdump(map, &tsiz);
   cbmapclose(map);
   if(_qdbm_deflate){
-    if(!(zbuf = _qdbm_deflate(tmp, tsiz, &zsiz))){
+    if(!(zbuf = _qdbm_deflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
       dpecode = DP_EMISC;
       odeum->fatal = TRUE;
@@ -321,7 +321,7 @@ int odoutbyid(ODEUM *odeum, int id){
     return FALSE;
   }
   if(_qdbm_inflate){
-    if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz))){
+    if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
       dpecode = DP_EBROKEN;
       odeum->fatal = TRUE;
@@ -396,7 +396,7 @@ ODDOC *odgetbyid(ODEUM *odeum, int id){
     return NULL;
   }
   if(_qdbm_inflate){
-    if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz))){
+    if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
       dpecode = DP_EBROKEN;
       odeum->fatal = TRUE;
