@@ -29,6 +29,9 @@ extern "C" {
 namespace qdbm {
   class DBM_error;
   class Datum;
+  Datum operator +(const Datum& former, const Datum& latter);
+  Datum operator +(const Datum& datum, const char* str);
+  Datum operator +(const char* str, const Datum& datum);
   class ADBM;
   class Depot_error;
   class Depot;
