@@ -1059,7 +1059,7 @@ int vlrepair(const char *name, VLCFUNC cmp){
     if(ksiz == sizeof(int) && *(int *)kbuf < VL_NODEIDMIN && *(int *)kbuf > 0){
       if((vbuf = dpget(depot, (char *)kbuf, sizeof(int), 0, -1, &vsiz)) != NULL){
         if(_qdbm_inflate && (flags & VL_FLISZLIB) &&
-           (zbuf = _qdbm_inflate(vbuf, vsiz, &zsiz)) != NULL){
+           (zbuf = _qdbm_inflate(vbuf, vsiz, &zsiz, _QDBM_ZMRAW)) != NULL){
           free(vbuf);
           vbuf = zbuf;
           vsiz = zsiz;
@@ -1440,7 +1440,7 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
     }
   }
   if(_qdbm_deflate && villa->zmode){
-    if(!(zbuf = _qdbm_deflate(CB_DATUMPTR(buf), CB_DATUMSIZE(buf), &zsiz))){
+    if(!(zbuf = _qdbm_deflate(CB_DATUMPTR(buf), CB_DATUMSIZE(buf), &zsiz, _QDBM_ZMRAW))){
       cbdatumclose(buf);
       if(dpecode == DP_EMODE) dpecodeset(DP_EMISC, __FILE__, __LINE__);
       return FALSE;
@@ -1491,7 +1491,7 @@ static VLLEAF *vlleafload(VILLA *villa, int id){
     return NULL;
   }
   if(_qdbm_inflate && villa->zmode){
-    if(!(zbuf = _qdbm_inflate(buf, size, &zsiz))){
+    if(!(zbuf = _qdbm_inflate(buf, size, &zsiz, _QDBM_ZMRAW))){
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       free(buf);
       return NULL;
